/*
 *  Authors: pbrufal (and tican)
 *
 *  Declarations for Syscriptor.
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <arpa/inet.h>
#include <net/if.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/utsname.h>
#include <unistd.h>

void network(void)
{
	struct ifconf ifc;
	struct ifreq *ifr=NULL;
	struct utsname name;
	FILE *proc;
	char buf[100], buff[8192];
	char *card, *card2, *card3;
	char *iname, *myip, *mynetmask;
	char *fname = "/proc/pci";
	int fd, i, n;
        int total = 0;

        proc = fopen(fname, "r");
        if ( !proc ) {
                fprintf(stderr, "Could not open %s. Bailing out.\n", fname);
                return;
        }

	puts("");
	printf("  \e[0;07m");
	printf("NETWORK\n\n");
	printf("\e[0;39m");

	printf("\e[1;37m  Network card:\e[0;39m\n");

	while ( fgets(buf, sizeof buf, proc) ) {
		card = strstr(buf,"Ethernet");
		if ( card ) {
			card2 = strchr(card, '(');
			if ( card2 ) {
				card2[-1] = 0;
				card3 = strchr(card,':')+1;
				printf(" %s\n", card3);
			}
		}
	}
	fclose(proc);

	puts("");

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
		return;
	}
	ifc.ifc_len = sizeof(buff);
	ifc.ifc_buf = buff;
	if (ioctl(fd, SIOCGIFCONF, &ifc) != 0) {
		close(fd);
		return;
	} 
	ifr = ifc.ifc_req;
	n = ifc.ifc_len / sizeof(struct ifreq);

	printf("\e[1;37m  Interfaces:\e[0;39m\n");

	for (i=n-1; i>=0 && total < 8; i--) {
		if ( ioctl(fd, SIOCGIFADDR, &ifr[i]) != 0 ) {
			continue;
		}
		iname = ifr[i].ifr_name;
		printf("\n\e[1;37m  %s:\e[0;39m\n",iname);
		myip = inet_ntoa( ((struct sockaddr_in *)&ifr[i].ifr_addr)->sin_addr );
		printf("\tIP:\t\t%s\n",myip);
		if ( ioctl(fd, SIOCGIFFLAGS, &ifr[i]) != 0 ) {
			continue;
		}
		if ( !(ifr[i].ifr_flags & IFF_UP) ) {
			continue;
		}
		if ( ioctl(fd, SIOCGIFNETMASK, &ifr[i]) != 0 ) {
			continue;
		}
 		mynetmask = inet_ntoa(((struct sockaddr_in *)&ifr[i].ifr_addr)->sin_addr);
		printf("\tNetmask:\t%s\n",mynetmask);
		total++;
	}
	close(fd);

	puts("");

	if (uname(&name) != -1) {
		printf("\e[1;37m  Hostname:\e[0;39m\n");
		printf("  %s\n", name.nodename);
	} else {
		fprintf(stderr, "  Could not find your hostname\n");
		exit(1);
	}

	puts("");
}

